@@ -15,7 +15,6 @@ package ClamTk::App;
 # use warnings FATAL => 'all';
 $| = 1;
 
-use Net::DNS;
 use Date::Calc 'Date_to_Days';
 use File::Basename 'basename';
 
@@ -24,7 +23,7 @@ use Locale::gettext;
 
 sub get_TK_version {
     # Stick with %.2f format - 4.50 vice 4.5
-    return '4.38';
+    return '4.41';
 }
 
 sub get_path {
@@ -69,6 +68,13 @@ sub get_path {
         : ( -e '/opt/local/bin/freshclam' ) ? '/opt/local/bin/freshclam'
         :                                     '';
 
+    # Use sigtool for db info
+    $path->{sigtool} =
+          ( -e '/usr/bin/sigtool' )       ? '/usr/bin/sigtool'
+        : ( -e '/usr/local/bin/sigtool' ) ? '/usr/local/bin/sigtool'
+        : ( -e '/opt/local/bin/sigtool' ) ? '/opt/local/bin/sigtool'
+        :                                   '';
+
     # Most times clamscan is under /usr/bin
     # We'll use clampath as the actual path
     # and clamscan as clampath + scan options
@@ -120,25 +126,43 @@ sub def_paths {
 
     # If the user selected "manual", that directory needs
     # to be checked first, so we'll jam that in with unshift.
-    my $user_set = 0;
-    if ( ClamTk::Prefs->get_preference('Update') eq 'single' ) {
+    my $user_set      = 0;
+    my $update_method = ClamTk::Prefs->get_preference('Update');
+    if ( $update_method eq 'single' ) {
         $user_set = 1;
         my $paths = ClamTk::App->get_path('db');
         unshift( @dirs, $paths );
     }
 
     # We'll search for the daily file then main;
-    # and check for .cld before .cvd
+    # Check for daily's .cld before .cvd,
+    # but main's .cvd before .cld
+    my $dupe_db = ClamTk::Prefs->get_preference('DupeDB');
     for my $dir_list (@dirs) {
+        # Check for duplicate daily databases
+        if ( -e "$dir_list/daily.cld" && -e "$dir_list/daily.cvd" ) {
+            only_one("$dir_list")
+                if ( ( $dupe_db && $update_method eq 'single' )
+                or ( $dupe_db && $> == 0 ) );
+        }
+
         if ( -e "$dir_list/daily.cld" ) {
             $DAILY_PATH = $dir_list;
         } elsif ( -e "$dir_list/daily.cvd" ) {
             $DAILY_PATH = $dir_list;
         }
-        if ( -e "$dir_list/main.cld" ) {
-            $MAIN_PATH = "$dir_list/main.cld";
-        } elsif ( -e "$dir_list/main.cvd" ) {
+
+        # Check for duplicate main databases
+        if ( -e "$dir_list/main.cld" && "$dir_list/main.cvd" ) {
+            unlink("$dir_list/main.cld")
+                if ( ( $dupe_db && $update_method eq 'single' )
+                or ( $dupe_db && $> == 0 ) );
+        }
+
+        if ( -e "$dir_list/main.cvd" ) {
             $MAIN_PATH = "$dir_list/main.cvd";
+        } elsif ( -e "$dir_list/main.cld" ) {
+            $MAIN_PATH = "$dir_list/main.cld";
         }
         last if ( $DAILY_PATH && $MAIN_PATH );
 
@@ -149,6 +173,57 @@ sub def_paths {
     return ( $DAILY_PATH, $MAIN_PATH );
 }
 
+sub only_one {
+    my ($location) = shift;
+    my ( $cld, $cvd ) = ('01 Jan 1900') x 2;
+    my $sigtool = get_path( undef, 'sigtool' );
+
+    if ( open( my $CLD, '-|', "$sigtool -i $location/daily.cld" ) ) {
+        while (<$CLD>) {
+            if (/Build time: (\d+\s\w+\s\d{4})/) {
+                $cld = $1;
+                last;
+            }
+        }
+    } else {
+        # shouldn't happen
+        $cld = '01 01 1970';
+    }
+    # warn "in only_one: cld = >$cld<\n";
+
+    if ( open( my $CVD, '-|', "$sigtool -i $location/daily.cvd" ) ) {
+        while (<$CVD>) {
+            if (/Build time: (\d+\s\w+\s\d{4})/) {
+                $cvd = $1;
+                last;
+            }
+        }
+    } else {
+        # shouldn't happen
+        $cvd = '01 01 1970';
+    }
+    # warn "in only_one: cvd = >$cvd<\n";
+
+    my $cmp = comp_dates( $cld, $cvd );
+    # If cmp == -1, cvd is newer.
+    # If cmp ==  1, cld is newer.
+    # If cmp ==  0, they're the same.
+    if ( $cmp == -1 ) {
+        # warn "cvd is newer\n";
+        unlink("$location/daily.cld")
+            or warn "Cannot delete $location/daily.cld: $!\n";
+    } elsif ( $cmp == 1 ) {
+        # warn "cld is newer\n";
+        unlink("$location/daily.cvd")
+            or warn "Cannot delete $location/daily.cvd: $!\n";
+    } elsif ( $cmp == 0 ) {
+        # warn "the same\n";
+        unlink("$location/daily.cvd")
+            or warn "Cannot delete $location/daily.cvd: $!\n";
+    }
+    return;
+}
+
 sub get_AV_version {
     # simple 'clamscan -V'.
     # We have to parse something like this:
@@ -171,50 +246,6 @@ sub get_AV_version {
     return $version ? $version : '0.00';
 }
 
-sub get_AV_remote {
-    # The user may have set the preference to not check this
-    # on startup using 'AVCheck'.  We'll let the user manually
-    # check anyway if using the update checker, though
-    my $call = ( caller(1) )[0];
-    return 'undef'
-        if ( !ClamTk::Prefs->get_preference('AVCheck')
-        && $call eq 'ClamTk::GUI' );
-
-    # Host for version
-    my $rmt_host = 'current.cvd.clamav.net';
-
-    # No longer need the alarm to "die" since Net::DNS
-    # does it for us.
-    # Just to be sure, we'll set a UDP timeout.
-    # 3 seconds ought to be enough for anyone.
-    my ( $get, $res );
-
-    $res = Net::DNS::Resolver->new( udp_timeout => 3, );
-    # We have permission from OpenDNS to use this. :)
-    if ( ClamTk::Prefs->get_preference('OpenDNS') ) {
-        $res->nameservers(
-            qw(208.67.222.222 208.67.220.220
-                208.67.222.220 208.67.220.222)
-                );
-    }
-
-    ($get) = $res->send( $rmt_host, "TXT" );
-    if ( !$get ) {
-        warn $res->errorstring, "\n";
-        return 'undef';
-    }
-    chomp($get);
-
-  # This is a sample of the line that is returned ($get->string):
-  # current.cvd.clamav.net.	439	IN	TXT	"0.95.3:51:10091:1259420373:1:44:12728"
-  # We just need the version, held between the " and the :
-    if ( $get->string =~ /IN\s+TXT\s+\"([\d\.]+):/ ) {
-        return $1;
-    } else {
-        return 'undef';
-    }
-}
-
 sub get_num_sigs {
     # Adds the main + daily for total # of signatures
     my ( undef, $daily, $main ) = get_sig_info();
@@ -224,13 +255,15 @@ sub get_num_sigs {
 sub get_date_sigs {
     # Gets the date, preferably from daily.c?d.
     # Useful for informing user of outdated sigs.
-    my ( $date, undef, undef ) = get_sig_info();
+    #my ( $date, undef, undef ) = get_sig_info( );
+    my ($date) = get_sig_info('date-only');
     return $date;
 }
 
 sub get_sig_info {
     # This sub parses the signature files for the date
     # and number of signatures.
+    my $date_only = shift || '';
     my ( $daily_path, $main_path ) = def_paths();
 
     # warn "using >$daily_path< for definitions.\n";
@@ -240,7 +273,10 @@ sub get_sig_info {
     # $INFO_DATE = Date of signatures
     # $INFO_DAILY = # of 'daily' signatures (not part of main.{cld,cvd})
     # $INFO_MAIN = # of signatures part of 'main' signatures
-    my ( $INFO_DATE, $INFO_DAILY, $INFO_MAIN ) = (0) x 3;
+    my ( $INFO_DATE, $INFO_DAILY,  $INFO_MAIN )   = (0) x 3;
+    my ( $MAIN_V,    $DAILY_CLD_V, $DAILY_CVD_V ) = (0) x 3;
+
+    my $sigtool = get_path( undef, 'sigtool' );
 
     # example of what we need to parse
     # ClamAV-VDB:07 Nov 2009 03-23 -0500:9999:102368:44:X:X:ccordes:1257582210
@@ -248,18 +284,22 @@ sub get_sig_info {
     # We'll store the date of main.{cld,cvd}; if daily.{cld,cvd} does not
     # exist, use that date as the date of signatures instead of returning
     # 'None found'. If daily.{cld,cvd} does exist, use that date instead.
-    if ($main_path) {
-        my $FILE;
-        if ( open( $FILE, '<', $main_path ) ) {
-            binmode($FILE);
-            while (<$FILE>) {
-                if (/ClamAV-VDB:(\S+\s+\S+\s+\S+).*?[\-\+]\d+:\d+:(\d+)/) {
+    if ( !$date_only && $main_path ) {
+        if ( open( my $main_db, '-|', "$sigtool -i $main_path" ) ) {
+            while (<$main_db>) {
+                chomp;
+                if (/Build time:\s(\d+\s\w+\s\d{4})/i) {
                     $INFO_DATE = $1;
-                    $INFO_MAIN = $2;
-                    last;
                 }
-                close($FILE)
-                    or warn "Couldn't close $main_path: $!\n";
+                if (/Signatures:\s(\d+)/) {
+                    $INFO_MAIN = $1;
+                    #last;
+                }
+                if (/Verification OK/) {
+                    $MAIN_V = 1;
+                }
+                #close($main_db)
+                #    or warn "Couldn't close $main_path: $!\n";
             }
         } else {
             $INFO_MAIN = 0;
@@ -272,47 +312,53 @@ sub get_sig_info {
     my ( $cvd_day, $cvd_mon, $cvd_year );
 
     # Check for .cld
-    my $FILE;
-    if ( open( $FILE, '<', "$daily_path/daily.cld" ) ) {
-        binmode($FILE);
-        while (<$FILE>) {
-            if (/ClamAV-VDB:(\S+\s+\S+\s+\S+).*[\-\+]\d+:\d+:(\d+)/) {
-                ( $cld_day, $cld_mon, $cld_year ) = split( /\s+/, $1 );
-                $INFO_DAILY = $2;
-                last;
+    if ( -e "$daily_path/daily.cld" ) {
+        if ( open( my $cld_db, '-|', "$sigtool -i $daily_path/daily.cld" ) ) {
+            while (<$cld_db>) {
+                if (/Build time:\s(\d+\s\w+\s\d{4})/i) {
+                    ( $cld_day, $cld_mon, $cld_year ) = split( /\s+/, $1 );
+                }
+                if (/Signatures:\s(\d+)/) {
+                    $INFO_DAILY = $1;
+                }
             }
+            #close($cld_db)
+            #    or warn "Couldn't close $daily_path (cld): $!\n";
         }
-        close($FILE)
-            or warn "Couldn't close $daily_path: $!\n";
     }
     if ( !$cld_day ) {
         $cld_day  = '01';
         $cld_mon  = 'Jan';
         $cld_year = '1900';
     }
-    # warn "cld: $cld_day $cld_mon $cld_year\n";
 
     # Check for .cvd
-    if ( open( $FILE, '<', "$daily_path/daily.cvd" ) ) {
-        binmode($FILE);
-        while (<$FILE>) {
-            if (/ClamAV-VDB:(\S+\s+\S+\s+\S+).*[\-\+]\d+:\d+:(\d+)/) {
-                ( $cvd_day, $cvd_mon, $cvd_year ) = split( /\s+/, $1 );
-                $INFO_DAILY = $2;
-                last;
+    if ( -e "$daily_path/daily.cvd" ) {
+        if ( open( my $cvd_db, '-|', "$sigtool -i $daily_path/daily.cvd" ) ) {
+            while (<$cvd_db>) {
+                if (/Build time:\s(\d+\s\w+\s\d{4})/i) {
+                    ( $cvd_day, $cvd_mon, $cvd_year ) = split( /\s+/, $1 );
+                }
+                if (/Signatures:\s(\d+)/) {
+                    $INFO_DAILY = $1;
+                }
             }
+            #close($cvd_db)
+            #    or warn "Couldn't close $daily_path (cvd): $!\n";
         }
-        close($FILE)
-            or warn "Couldn't close $daily_path: $!\n";
     }
     if ( !$cvd_day ) {
         $cvd_day  = '01';
         $cvd_mon  = 'Jan';
         $cvd_year = '1900';
     }
-    # warn "cvd: $cvd_day $cvd_mon $cvd_year\n";
 
-    # Now compare the dates and return the more recent one.
+    # If cmp == -1, cvd is newer.
+    # If cmp ==  1, cld is newer.
+    # If cmp ==  0, they're the same.
+    my $cmp = comp_dates( "$cld_day $cld_mon $cld_year",
+        "$cvd_day $cvd_mon $cvd_year" );
+
     my %months = (
         'Jan' => '01',
         'Feb' => '02',
@@ -328,21 +374,15 @@ sub get_sig_info {
         'Dec' => 12,
         );
 
-    my $cmp =
-        ( Date_to_Days( $cld_year, $months{$cld_mon}, $cld_day )
-            <=> Date_to_Days( $cvd_year, $months{$cvd_mon}, $cvd_day ) );
-
-    # If cmp == -1, cvd is newer.
-    # If cmp ==  1, cld is newer.
-    # If cmp ==  0, they're the same.
-
     $INFO_DATE =
           ( $cmp == -1 ) ? join( ' ', $cvd_day, $months{$cvd_mon}, $cvd_year )
         : ( $cmp == 0 )  ? join( ' ', $cvd_day, $months{$cvd_mon}, $cvd_year )
         : ( $cmp == 1 )  ? join( ' ', $cld_day, $months{$cld_mon}, $cld_year )
         :                  '01 01 1900';
 
-    return ( $INFO_DATE, $INFO_DAILY, $INFO_MAIN );
+    return ($date_only)
+        ? $INFO_DATE
+        : ( $INFO_DATE, $INFO_DAILY, $INFO_MAIN );
 }
 
 sub lastscan {
@@ -365,6 +405,37 @@ sub lastscan {
     return "$day $month $year";
 }
 
+sub comp_dates {
+    my ( $cld, $cvd ) = @_;
+    my %months = (
+        'Jan' => '01',
+        'Feb' => '02',
+        'Mar' => '03',
+        'Apr' => '04',
+        'May' => '05',
+        'Jun' => '06',
+        'Jul' => '07',
+        'Aug' => '08',
+        'Sep' => '09',
+        'Oct' => 10,
+        'Nov' => 11,
+        'Dec' => 12,
+        );
+
+    my ( $cld_day, $cld_mon, $cld_year ) = split( /\s/, $cld );
+    my ( $cvd_day, $cvd_mon, $cvd_year ) = split( /\s/, $cvd );
+
+    my $cmp =
+        ( Date_to_Days( $cld_year, $months{$cld_mon}, $cld_day )
+            <=> Date_to_Days( $cvd_year, $months{$cvd_mon}, $cvd_day ) );
+
+    # If cmp == -1, cvd is newer.
+    # If cmp ==  1, cld is newer.
+    # If cmp ==  0, they're the same.
+
+    return $cmp;
+}
+
 sub translate {
     # This is a dummy routine, solely for the .desktop file.
     return gettext('Scan for viruses...');
