@@ -20,7 +20,7 @@ package ClamTk::Device;
 # So, we'll use udev.  We're not going to monitor anything; rather,
 # check what udev is reporting, and then see if it has a mountpoint.
 
-use strict;
+# use strict;
 # use warnings FATAL => 'all';
 $| = 1;
 
