@@ -25,7 +25,6 @@ use Date::Calc 'Delta_Days';
 use Digest::MD5 'md5_hex';
 use Locale::gettext;
 use LWP::UserAgent;
-use File::Find::Rule;
 use Encode;
 use URI;
 
@@ -43,9 +42,6 @@ my $window;    # Top level window
 
 my $directive = '';    # Arguments passed to clamscan
 my %dirs_scanned;      # Keeps track of directories scanned for display
-my $pb;                # Progressbar
-my $step    = 0;       # ...with the progressbar ($pb)
-my $current = 0;       # ...with the progressbar ($pb)
 my $SCAN;              # Scan handle; must be global for stop button
 my $scan_pid;          # Pid of scan handle; must be global for stop button
 my $stopped = 1;       # Program is stopped if 1, running if 0
@@ -56,7 +52,6 @@ my $scan_frame;        # Holds the progressbar, files scanned, etc.
 my $drop_frame;        # For the drag and drop stuff
 my $found_count = 0;   # Number of threats found
 my $num_scanned = 0;   # Number of files scanned, clean or bad
-my $av_remote   = '';  # ClamAV version, according to remote servers
 my $proxy_status_img;       # Image; global for 'flicker' appearance
 my ($scan_status_label);    # Shows what's being scanned, a little more
 
@@ -68,7 +63,7 @@ my $white = Gtk2::Gdk::Color->new( 0xFFFF, 0xFFFF, 0xFFFF );
 my $last_scan_bar = Gtk2::Label->new;
 
 # Scan frame stuff, under the progressbar
-my ( $left_status, $mid_status, $right_status );
+my ( $left_status, $mid_status );
 
 # Right-most column
 my ( $engine_version_text, $defs_version_text );
@@ -76,11 +71,31 @@ my ( $engine_version_text, $defs_version_text );
 # Left-most column - global so we can adjust them throughout
 my ( $engine_gui_img, $defs_gui_img, $status_gui_img );
 
+# $stop_btn is global because its tooltips change based on
+# whether or not ClamTk is actively scanning
+my $stop_btn;
+
+# $loading_img is the spinner button.
+my $loading_img;
+# Most packagers are going to miss the inclusion of $loading_img,
+# or they'll miss the new "images/" directory, so we'll decide
+# whether or not to show it based on whether it's where we expect
+my $show_loader = 1;
+
+# Assuming images are under /usr/share/pixmaps.  Keeping the
+# directory up here so packagers can change it.
+my $images_dir = '/usr/share/pixmaps';
+
+# Tooltips
+my $tt = '';
+
 my ( $new_slist, $new_hlist );
 
 sub start_gui {
     INIT { Gtk2->init; }    ## no critic
 
+    $tt = Gtk2::Tooltips->new();
+
     # The main window
     $window = Gtk2::Window->new();
     $window->signal_connect( destroy => sub { Gtk2->main_quit; } );
@@ -88,10 +103,10 @@ sub start_gui {
     $window->set_border_width(0);
     $window->set_resizable(FALSE);
     $window->set_position('center');
-    if ( -e '/usr/share/pixmaps/clamtk.png' ) {
-        $window->set_default_icon_from_file('/usr/share/pixmaps/clamtk.png');
-    } elsif ( -e '/usr/share/pixmaps/clamtk.xpm' ) {
-        $window->set_default_icon_from_file('/usr/share/pixmaps/clamtk.xpm');
+    if ( -e "$images_dir/clamtk.png" ) {
+        $window->set_default_icon_from_file("$images_dir/clamtk.png");
+    } elsif ( -e "$images_dir/clamtk.xpm" ) {
+        $window->set_default_icon_from_file("$images_dir/clamtk.xpm");
     }
 
     my $vbox = Gtk2::VBox->new( FALSE, 0 );
@@ -193,7 +208,7 @@ sub start_gui {
         ],
 
         ### These fall under Help ###
-        # Dialog box to update signatures, check for GUI updates
+        # Dialog box for Help, to update signatures, check for GUI updates.
         [   'UpdateSig', 			undef,
             gettext('_Check for updates'), 	'<control>U',
 	    undef,	sub { ClamTk::Update->update_dialog($window->get_position) },
@@ -281,9 +296,6 @@ sub start_gui {
     $ui->add_ui_from_string($ui_info);
     $vbox->pack_start( $ui->get_widget('/MenuBar'), FALSE, FALSE, 0 );
 
-    # We have to declare this now for items in the Actions frame
-    my $tt = Gtk2::Tooltips->new();
-
     # Last scan bar area
     # Shows things like "Please wait..."
     # We have to put it at the top now -
@@ -365,18 +377,6 @@ sub start_gui {
     my $status_table = Gtk2::Table->new( 3, 3, FALSE );
     $status_frame->add($status_table);
 
-    # Version of ClamAV
-    my $engine_gui_box = Gtk2::HBox->new( TRUE, 0 );
-    my $engine_text = Gtk2::Label->new( gettext('Antivirus engine') );
-    $engine_version_text = Gtk2::Label->new( gettext('Unknown') );
-    $engine_gui_img =
-        Gtk2::Image->new_from_stock( 'gtk-yes', 'small-toolbar' );
-    $tt->set_tip( $engine_gui_img, gettext('Current') );
-    $engine_gui_box->pack_start( $engine_gui_img,      TRUE, TRUE, 0 );
-    $engine_gui_box->pack_start( $engine_text,         TRUE, TRUE, 0 );
-    $engine_gui_box->pack_start( $engine_version_text, TRUE, TRUE, 0 );
-    $status_table->attach_defaults( $engine_gui_box, 0, 3, 0, 1 );
-
     # Version of ClamTk
     my $status_gui_box   = Gtk2::HBox->new( TRUE, 0 );
     my $gui_text         = Gtk2::Label->new( gettext('GUI version') );
@@ -387,7 +387,7 @@ sub start_gui {
     $status_gui_box->pack_start( $status_gui_img,   TRUE, TRUE, 0 );
     $status_gui_box->pack_start( $gui_text,         TRUE, TRUE, 0 );
     $status_gui_box->pack_start( $gui_version_text, TRUE, TRUE, 0 );
-    $status_table->attach_defaults( $status_gui_box, 0, 3, 1, 2 );
+    $status_table->attach_defaults( $status_gui_box, 0, 3, 0, 1 );
 
     # Date of antivirus signatures
     my $defs_gui_box = Gtk2::HBox->new( TRUE, 0 );
@@ -398,7 +398,19 @@ sub start_gui {
     $defs_gui_box->pack_start( $defs_gui_img,      TRUE, TRUE, 0 );
     $defs_gui_box->pack_start( $defs_text,         TRUE, TRUE, 0 );
     $defs_gui_box->pack_start( $defs_version_text, TRUE, TRUE, 0 );
-    $status_table->attach_defaults( $defs_gui_box, 0, 3, 2, 3 );
+    $status_table->attach_defaults( $defs_gui_box, 0, 3, 1, 2 );
+
+    # Version of ClamAV
+    my $engine_gui_box = Gtk2::HBox->new( TRUE, 0 );
+    my $engine_text = Gtk2::Label->new( gettext('Antivirus engine') );
+    $engine_version_text = Gtk2::Label->new( gettext('Unknown') );
+    $engine_gui_img =
+        Gtk2::Image->new_from_stock( 'gtk-info', 'small-toolbar' );
+    $tt->set_tip( $engine_gui_img, gettext('Version') );
+    $engine_gui_box->pack_start( $engine_gui_img,      TRUE, TRUE, 0 );
+    $engine_gui_box->pack_start( $engine_text,         TRUE, TRUE, 0 );
+    $engine_gui_box->pack_start( $engine_version_text, TRUE, TRUE, 0 );
+    $status_table->attach_defaults( $engine_gui_box, 0, 3, 2, 3 );
 
     # Scan frame:
     # This holds the progressbar and keeps track of files
@@ -415,28 +427,43 @@ sub start_gui {
     $scan_status_label = Gtk2::Label->new('');
     $scan_status_label->set_justify('center');
     $scan_status_label->set_ellipsize('middle');
-    $scan_box->pack_start( $scan_status_label, FALSE, FALSE, 2 );
 
-    my $scan_table = Gtk2::Table->new( 1, 2, FALSE );
+    my $scan_table = Gtk2::Table->new( 1, 3, FALSE );
     $scan_box->pack_start( $scan_table, FALSE, FALSE, 0 );
 
-    $pb = Gtk2::ProgressBar->new;
-    $scan_table->attach( $pb, 0, 1, 0, 1, [ 'fill', 'expand' ],
+    $scan_table->attach( $scan_status_label, 0, 1, 0, 1, [ 'fill', 'expand' ],
         ['shrink'], 0, 0 );
-    $pb->set_fraction(0);
+
+    if ( -e "$images_dir/clamtk-loader.gif" ) {
+        $loading_img =
+            Gtk2::Image->new_from_file("$images_dir/clamtk-loader.gif");
+        $scan_table->attach( $loading_img, 1, 2, 0, 1, [ 'shrink', 'shrink' ],
+            ['shrink'], 0, 0 );
+        $show_loader = 1;
+    } else {
+        $loading_img = Gtk2::Image->new_from_stock( 'gtk-missing-image',
+            'small-toolbar' );
+        $scan_table->attach( $loading_img, 1, 2, 0, 1, [ 'shrink', 'shrink' ],
+            ['shrink'], 0, 0 );
+        $show_loader = 0;
+    }
 
     my $stop_image =
         Gtk2::Image->new_from_stock( 'gtk-stop', 'small-toolbar' );
-    my $stop_btn = Gtk2::Button->new();
+    $stop_btn = Gtk2::Button->new();
     $stop_btn->set_property( image => $stop_image );
     $stop_btn->set_relief('none');
     $tt->set_tip( $stop_btn, gettext('Stop scanning now') );
-    $scan_table->attach( $stop_btn, 1, 2, 0, 1, [ 'shrink', 'shrink' ],
+    $scan_table->attach( $stop_btn, 2, 3, 0, 1, [ 'shrink', 'shrink' ],
         ['shrink'], 0, 0 );
     $stop_btn->signal_connect(
         'clicked' => sub {
             clear_output(), return if ($stopped);
+            $loading_img->hide()
+                if ( $loading_img->visible );
 
+            kill 15, $scan_pid + 1;
+            waitpid( $scan_pid + 1, 0 ) if ( $scan_pid + 1 );
             kill 15, $scan_pid if ($scan_pid);
             waitpid( $scan_pid, 0 ) if ($scan_pid);
             please_wait();
@@ -470,9 +497,6 @@ sub start_gui {
     $mid_status = Gtk2::Label->new( gettext('Threats Found: ') );
     $bottom_box->pack_start( $mid_status, TRUE, TRUE, 5 );
 
-    $right_status = Gtk2::Label->new( gettext('Ready') );
-    $bottom_box->pack_start( $right_status, TRUE, TRUE, 5 );
-
     # Drag-n-drop frame
     $drop_frame = Gtk2::Frame->new();
     $vbox->pack_start( $drop_frame, TRUE, TRUE, 2 );
@@ -527,6 +551,7 @@ sub start_gui {
 
     # The only reason we take @ARGV is for right-click scanning,
     # not arguments (e.g., -v).
+    please_wait();
     if (@ARGV) {
         update_status_frame();
         if ( -d $ARGV[0] ) {
@@ -622,16 +647,10 @@ sub start_gui {
 sub update_status_frame {
     # This is useful as its own subroutine so we
     # can call it from anywhere
-    $last_scan_bar->set_markup( "<span background = '#FFFFC8'>"
-            . gettext('Please wait, initializing...')
-            . "</span>" );
-    $last_scan_bar->show_all();
     Gtk2->main_iteration while ( Gtk2->events_pending );
 
     # ClamAV version
-    set_AV_engine_status();
-
-    my $tt = Gtk2::Tooltips->new();
+    $engine_version_text->set_text( ClamTk::App->get_AV_version() );
 
     # ClamTk version
     Gtk2->main_iteration while ( Gtk2->events_pending );
@@ -659,51 +678,6 @@ sub update_status_frame {
     return;
 }
 
-sub set_AV_engine_status {
-    my ($version) = ClamTk::App->get_AV_version();
-    $engine_version_text->set_text($version);
-
-    # Check for latest version
-    $av_remote = ClamTk::App->get_AV_remote();
-
-    my $tt = Gtk2::Tooltips->new();
-
-    if ( $av_remote eq 'undef' || $av_remote eq 'unavailable' ) {
-        $engine_gui_img->set_from_stock( 'gtk-dialog-question',
-            'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Unable to check') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-        return ( gettext('Unable to check') );
-    } elsif ( ( $version cmp $av_remote ) == -1 ) {
-        $engine_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Outdated') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-        return ( gettext('Outdated') );
-    } elsif ( ( $version cmp $av_remote ) == 0 ) {
-        $engine_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Current') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-        return ( gettext('Current') );
-    } elsif ( ( $version cmp $av_remote ) == 1 ) {    # shouldn't happen
-        $engine_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Current') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-        return ( gettext('Current') );
-    } else {
-        $engine_gui_img->set_from_stock( 'gtk-dialog-question',
-            'small-toolbar' );
-        $tt->set_tip( $engine_gui_img, gettext('Unable to check') );
-        $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-        return ( gettext('Unable to check') );
-    }
-    return;
-}
-
 sub set_sig_status {
     if ( !$defs_version_text ) {
         # This may get called from Update.pm directly
@@ -724,8 +698,6 @@ sub set_sig_status {
     my ( $d, $m, $y ) = split / /, $sig_date;
     my $date_ret = date_diff( $d, $m, $y );
 
-    my $tt = Gtk2::Tooltips->new();
-
     if ( $date_ret eq 'outdated' || !$sig_date ) {
         $defs_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
         $defs_version_text->set_text( gettext('Outdated') );
@@ -746,8 +718,6 @@ sub set_sig_status {
 sub set_tk_status {
     my ( undef, $ret ) = @_;
 
-    my $tt = Gtk2::Tooltips->new();
-
     if ( $ret == 4 || $ret == 5 ) {
         $status_gui_img->set_from_stock( 'gtk-dialog-question',
             'small-toolbar' );
@@ -814,8 +784,6 @@ sub getfile {
     warn "Cannot scan without clamscan!\n" unless $paths->{clampath};
     return unless $paths->{clampath};
 
-    # Set the pulsebar to 0
-    $pb->set_fraction(0);
     Gtk2->main_iteration while ( Gtk2->events_pending );
     clear_output();
     chdir( $paths->{directory} ) or chdir('/tmp');
@@ -823,21 +791,12 @@ sub getfile {
     my ( $dir, $dialog );
     Gtk2->main_iteration while ( Gtk2->events_pending );
 
-    # We now only use this to count files,
-    # which makes the progressbar look good. I'd feel bad,
-    # but it doesn't seem to be any (much) slower...
-    my $rule = File::Find::Rule->new;
-    # For counting the files:
-    my @files;
-
     # By default, we ignore .gvfs directories.
     # Once we figure out KDE's process, we'll include that too.
     if ( !ClamTk::Prefs->get_preference('Mounted') ) {
         # This returned a '0', so filter the results
         for my $m (qw(.gvfs smb4k)) {
             $directive .= " --exclude-dir=$m";
-            $rule->not(
-                File::Find::Rule->name($m)->directory->prune->discard );
         }
     }
 
@@ -852,8 +811,6 @@ sub getfile {
         # We just want the base directory name for F::F::R:
         $ignore = ( split( '/', $ignore ) )[-1];
         $directive .= " --exclude-dir=$full";
-        $rule->not(
-            File::Find::Rule->name("$ignore")->directory->prune->discard );
     }
 
     # Remove mail directories for now -
@@ -867,18 +824,8 @@ sub getfile {
         );
     for my $mailbox (@maildirs) {
         $directive .= " --exclude-dir=$mailbox";
-        $rule->not(
-            File::Find::Rule->name($mailbox)->directory->prune->discard );
     }
 
-    $rule->file;
-    $rule->exists;
-    $rule->readable;
-    $rule->maxdepth(1)
-        unless ( $option eq 'recur'
-        or $option eq 'full-home'
-        or $prefs{Recursive} );
-
     # A more descriptive title based on what we're after
     my $title =
           ( $option eq 'file' ) ? gettext('Select File')
@@ -900,7 +847,6 @@ sub getfile {
         please_wait();
         $window->queue_draw;
         Gtk2->main_iteration while ( Gtk2->events_pending );
-        @files = $rule->in( $paths->{directory} );
     } elsif ( $option eq 'file' ) {
         $dialog = Gtk2::FileChooserDialog->new(
             gettext($title), $window, 'open',
@@ -914,7 +860,6 @@ sub getfile {
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
             push( @scans, $dialog->get_filenames );
-            @files = $dialog->get_filenames;
             $dialog->destroy;
             $window->queue_draw;
         } else {
@@ -925,14 +870,12 @@ sub getfile {
         my $dir = $cmd_input[0];
         return if ( not $dir or not -d $dir );
         Gtk2->main_iteration while ( Gtk2->events_pending );
-        @files = $rule->in($dir);
         push( @scans, $dir );
     } elsif ( $option eq 'full-home' ) {
         please_wait();
         Gtk2->main_iteration while ( Gtk2->events_pending );
         push( @scans, $paths->{directory} );
         $directive .= ' --recursive';
-        @files = $rule->in( $paths->{directory} );
     } elsif ( $option eq 'dir' or $option eq 'recur' ) {
         $dialog = Gtk2::FileChooserDialog->new(
             gettext($title), $window,
@@ -964,15 +907,21 @@ sub getfile {
             $window->queue_draw;
             $dir ||= $paths->{directory};
             Gtk2->main_iteration while ( Gtk2->events_pending );
-            @files = $rule->in($dir);
+
             push( @scans, $dir );
 
             if ( !$prefs{Recursive} ) {
-                $directive .= ' --max-dir-recursion=1'
-                    if ( $option ne 'recur' );
-            } elsif ( $prefs{Recursive} || $option eq 'recur' ) {
+                if ( $option ne 'recur' ) {
+                    $directive .= ' --max-dir-recursion=1';
+                }
+            } elsif ( $prefs{Recursive} ) {
                 $directive .= ' --recursive';
             }
+
+            if ( $option eq 'recur' ) {
+                $directive .= ' --recursive';
+            }
+
         } else {
             $dialog->destroy;
             return;
@@ -982,7 +931,6 @@ sub getfile {
             please_wait();
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
-            @files = $rule->in( $cmd_input[0] );
             push( @scans, $cmd_input[0] );
             if ( !$prefs{Recursive} ) {
                 $directive .= ' --max-dir-recursion=1';
@@ -997,23 +945,26 @@ sub getfile {
                 push( @scans, $_ );    # if ( -f $_ );
             }
             $directive .= ' --max-dir-recursion=1';
-            @files = @scans;
         }
     } else {
         warn "Unknown option '$option'.\n";
     }
 
+    # Can take a minute for clamscan to get going...
+    please_wait();
+
     # close the drag and drop frame
     $drop_frame->hide();
     # open the scan frame for display
-    $scan_frame->show_all();
-    Gtk2->main_iteration while ( Gtk2->events_pending );
-    $window->queue_draw;
+    $scan_frame->show();
+    $loading_img->hide() if ( $loading_img->visible );
+
     Gtk2->main_iteration while ( Gtk2->events_pending );
 
+    $tt->set_tip( $stop_btn, gettext('Stop scanning now') );
+
     # start the timer - replaces the "Ready"
     $start_time = time if ( !$start_time );
-    $right_status->set_text( gettext('Elapsed time: ') );
 
     # we'll count this as ! $stopped
     $stopped = 0;
@@ -1035,14 +986,7 @@ sub getfile {
         scan( $directive, @scans );
     } else {
         # Pass the files to the filter first
-        filter( $option, \@files );
-
-        # The pulse of the progressbar
-        if ( scalar(@files) > 1 ) {
-            $step = 1 / scalar(@files);
-        } else {
-            $step = 1;
-        }
+        filter($option);
 
         # By default, 20Mb is the largest we go -
         # unless the preference is to ignore size.
@@ -1050,15 +994,13 @@ sub getfile {
             $directive .= ' --max-filesize=20M';
         }
 
-        timer();
         scan( $directive, @scans );
     }
     clean_up();
 }
 
 sub filter {
-    my ($opt)      = shift;
-    my ($file_ref) = @_;
+    my ($opt) = shift;
 
     my $paths = ClamTk::App->get_path('all');
 
@@ -1067,7 +1009,6 @@ sub filter {
         && $opt ne 'recur'
         && $opt ne 'full-home' ) {
         $directive .= ' --exclude="\/\."';
-        @$file_ref = grep { basename($_) !~ /^\./ } @$file_ref;
     }
 
     # symlinks:
@@ -1079,20 +1020,7 @@ sub filter {
         || ( $version cmp '0.97' ) == 1 ) {
         $directive .= ' --follow-dir-symlinks=1';
         $directive .= ' --follow-file-symlinks=1';
-        # Attempt to remove dangling symlinks
-        my $index = 0;
-        while ( $index <= $#{$file_ref} ) {
-            if ( -l $file_ref->[$index] && !-e $file_ref->[$index] ) {
-                splice( @$file_ref, $index, 1 );
-            } else {
-                $index++;
-            }
-        }
     }
-
-    # Remove empty files - ClamAV ignores them.
-    # Really, this only affects the counting.
-    @$file_ref = grep { -s $_ } @$file_ref;
 }
 
 sub scan {
@@ -1105,9 +1033,6 @@ sub scan {
     # Leave if we have no direct files or directory to scan
     return if ( !@get );
 
-    # Update the timer
-    timer();
-
     my $paths   = ClamTk::App->get_path('all');
     my $command = $paths->{clamscan};
 
@@ -1128,19 +1053,32 @@ sub scan {
     # for some distros.
     binmode( $SCAN, ':utf8:bytes' );
 
-    # Turn on Tooltips
-    my $tt = Gtk2::Tooltips->new();
+    # pulse  = mode we're in; activity or blank
+    # pulser = Glib::Timeout; gets removed after scan
+    # iter   = used for pulsing every other file
+    my ( $pulse, $pulser, $iter );
+    $pulse = ClamTk::Prefs->get_preference('PulseMode');
+    if ( $pulse eq 'activity' ) {
+        $pulser = Glib::Timeout->add(
+            100,
+            sub {
+                timer();
+                $window->queue_draw;
+                Gtk2->main_iteration while Gtk2->events_pending;
+                return TRUE;
+            } );
+    } else {
+        $iter = 0;
+    }
 
     Gtk2->main_iteration while Gtk2->events_pending;
     while (<$SCAN>) {
         Gtk2->main_iteration while Gtk2->events_pending;
-        if ( $av_remote eq 'undef' && /ClamAV engine is outdated/ ) {
-            $engine_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
-            $engine_version_text->set_text( gettext('Outdated') );
-            $tt->set_tip( $engine_gui_img, gettext('Outdated') );
-            $window->queue_draw;
-            Gtk2->main_iteration while ( Gtk2->events_pending );
-        }
+
+        # Show our spinner
+        $loading_img->show()
+            if ( !$loading_img->visible && $show_loader );
+
         next if (/^LibClamAV/);
         next if (/^\s*$/);
         $last_scan_bar->set_text('');
@@ -1165,8 +1103,7 @@ sub scan {
         chomp($file)   if ( defined $file );
         chomp($status) if ( defined $status );
         $scan_status_label->set_text( sprintf gettext('Scanning %s...'),
-            dirname($file) )
-            if ($file);
+            dirname($file) );
 
         # Lots of temporary things under /tmp/clamav;
         # we'll just ignore them.
@@ -1181,7 +1118,14 @@ sub scan {
         # $status is the "virus" name.
         $status =~ s/\s+FOUND$//;
 
-        timer();
+        # If we're not in activity mode, pulse every other file.
+        # You'll go blind looking at that thing going back and forth.
+        if ( !$pulse ) {
+            if ( $iter % 2 ) {
+                timer();
+            }
+            $iter++;
+        }
 
         # These aren't necessarily clean (despite the variable's name)
         # - we just don't want them counted as viruses
@@ -1217,38 +1161,28 @@ sub scan {
         }
         $num_scanned++;
 
-        # Pulse the progressbar
-        if ( $current + $step <= 0 || $current + $step >= 1.0 ) {
-            $current = .99;
-        } else {
-            $current += $step;
-        }
         Gtk2->main_iteration while ( Gtk2->events_pending );
-        $pb->set_fraction($current);
-        $pb->set_text( sprintf gettext('Percent complete: %2d'),
-            $current * 100 );
-        Gtk2->main_iteration while ( Gtk2->events_pending );
-
-        # Refresh timer stats
-        timer();
+        timer() if ( !$pulse );
     }
-    timer();
+
+    please_wait();
+    Gtk2->main_iteration while ( Gtk2->events_pending );
+    if ( !$pulse ) {
+        timer();
+    } else {
+        Glib::Source->remove($pulser) if ($pulse);
+    }
+    $loading_img->hide();
 
     # Done scanning - close filehandle and return to
     # getfile() and then to clean-up
     close($SCAN);    # or warn "Unable to close scanner! $!\n";
-    $pb->set_text( gettext('Percent complete: 100') );
-    Gtk2->main_iteration while ( Gtk2->events_pending );
 }
 
 sub clear_output {
     # Return if there are files still being scanned
     return if ( !$stopped );
 
-    # Reset the progressbar
-    $pb->set_fraction(0);
-    $pb->set_text('');
-
     # Clear the text
     $last_scan_bar->set_text('');
     $last_scan_bar->hide();
@@ -1260,7 +1194,6 @@ sub clear_output {
     # Reset the bottom labels
     $left_status->set_text( gettext('Files Scanned: ') );
     $mid_status->set_text( gettext('Threats Found: ') );
-    $right_status->set_text( gettext('Ready') );
 
     # Hide the scanning frame
     $scan_frame->hide();
@@ -1274,21 +1207,17 @@ sub clear_output {
 
 sub timer {
     Gtk2->main_iteration while ( Gtk2->events_pending );
-    my $now     = time;
-    my $seconds = $now - $start_time;
-    my $s       = sprintf '%02d', ( $seconds % 60 );
-    my $m       = sprintf '%02d', ( $seconds - $s ) / 60;
-    $right_status->set_text( sprintf gettext('Elapsed time: %s'), "$m:$s" );
     $left_status->set_text( sprintf gettext('Files Scanned: %d'),
         $num_scanned );
     $window->queue_draw;
-    Gtk2->main_iteration while ( Gtk2->events_pending );
-    return 1;
+    return TRUE;
 }
 
 sub clean_up {
-    # Clear the progressbar
-    $pb->set_fraction(1.0);
+    Gtk2->main_iteration while ( Gtk2->events_pending );
+    $window->queue_draw;
+
+    $tt->set_tip( $stop_btn, gettext('Close window') );
 
     # No files scanned?
     if ( $num_scanned == 0 ) {
@@ -1296,18 +1225,22 @@ sub clean_up {
                 . gettext('No files were scanned.')
                 . "</span>" );
         $last_scan_bar->show_all();
-    } else {
-        $last_scan_bar->set_text('');
-        $last_scan_bar->hide();
     }
 
-    # ...and remove the text
-    $pb->set_text("");
+    # Only show the please_wait if files were scanned.
+    # Otherwise the 'no files scanned' will immediately be removed.
+    $window->queue_draw;
+    Gtk2->main_iteration while ( Gtk2->events_pending );
 
     my $db_total = ClamTk::App->get_num_sigs();
     my $REPORT;    # filehandle for histories log
     my ( $mon, $day, $year ) = split / /, strftime( '%b %d %Y', localtime );
 
+    # And now we can hide it because we have the info.
+    if ($num_scanned) {
+        $last_scan_bar->hide();
+    }
+
     # Save date of scan
     if ( $found_count > 0 ) {
         ClamTk::Prefs->set_preference( 'LastInfection', "$day $mon $year" );
@@ -1324,7 +1257,8 @@ sub clean_up {
 
     # sort the directories scanned for display
     my @sorted = sort { $a cmp $b } keys %dirs_scanned;
-    if ( open $REPORT, '>>:encoding(UTF-8)', $virus_log ) {
+    #if ( open $REPORT, '>>:encoding(UTF-8)', $virus_log ) {
+    if ( open $REPORT, '>>', $virus_log ) {
         print $REPORT "\nClamTk, v", ClamTk::App->get_TK_version(), "\n",
             scalar localtime, "\n";
         print $REPORT sprintf gettext("ClamAV Signatures: %d\n"), $db_total;
@@ -1351,7 +1285,6 @@ sub clean_up {
         $mid_status->set_markup( sprintf gettext('<b>Threats Found: %d</b>'),
             $found_count );
     }
-    $right_status->set_text( gettext('Ready') );
     $window->queue_draw;
 
     # Set the minimum sizes for the two columns,
@@ -1397,7 +1330,6 @@ sub clean_up {
     $stopped      = 1;
     $start_time   = '';
     $directive    = '';
-    $current      = 0;
 }
 
 sub show_message_dialog {
@@ -1933,24 +1865,13 @@ sub preferences {
         } );
     $mounted_box->can_focus(FALSE);
 
-    my $startup_table = Gtk2::Table->new( 4, 1, TRUE );
+    my $startup_table = Gtk2::Table->new( 3, 1, TRUE );
     $nb->insert_page( $startup_table, gettext('Startup Preferences'), 1 );
 
-    my $av_box = Gtk2::CheckButton->new_with_label(
-        gettext('Check for AV Engine updates') );
-    $av_box->set_active(TRUE) if ( $prefs{AVCheck} );
-    $startup_table->attach_defaults( $av_box, 0, 1, 0, 1 );
-    $av_box->signal_connect(
-        toggled => sub {
-            ClamTk::Prefs->set_preference( 'AVCheck',
-                $av_box->get_active ? 1 : 0 );
-        } );
-    $av_box->can_focus(FALSE);
-
     my $gui_box =
         Gtk2::CheckButton->new_with_label( gettext('Check for GUI updates') );
     $gui_box->set_active(TRUE) if ( $prefs{GUICheck} );
-    $startup_table->attach_defaults( $gui_box, 0, 1, 1, 2 );
+    $startup_table->attach_defaults( $gui_box, 0, 1, 0, 1 );
     $gui_box->signal_connect(
         toggled => sub {
             ClamTk::Prefs->set_preference( 'GUICheck',
@@ -1958,16 +1879,19 @@ sub preferences {
         } );
     $gui_box->can_focus(FALSE);
 
-    my $dns_box =
-        Gtk2::CheckButton->new_with_label( gettext('Use OpenDNS servers') );
-    $dns_box->set_active(TRUE) if ( $prefs{OpenDNS} );
-    $startup_table->attach_defaults( $dns_box, 0, 1, 2, 3 );
-    $dns_box->signal_connect(
+    my $gui_delete_box = Gtk2::CheckButton->new_with_label(
+        gettext('Remove duplicate signature databases') );
+    $gui_delete_box->set_active(TRUE) if ( $prefs{DupeDB} );
+    $startup_table->attach_defaults( $gui_delete_box, 0, 1, 1, 2 );
+    $gui_delete_box->signal_connect(
         toggled => sub {
-            ClamTk::Prefs->set_preference( 'OpenDNS',
-                $dns_box->get_active ? 1 : 0 );
+            ClamTk::Prefs->set_preference( 'DupeDB',
+                $gui_delete_box->get_active ? 1 : 0 );
         } );
-    $dns_box->can_focus(FALSE);
+
+    my $gui_spacer1 = Gtk2::Label->new();
+    $startup_table->attach_defaults( $gui_spacer1, 0, 1, 2, 3 );
+    $gui_delete_box->can_focus(FALSE);
 
     my $whitebox = Gtk2::VBox->new( FALSE, 0 );
     $nb->insert_page( $whitebox, gettext('Whitelist'), 2 );
@@ -2362,7 +2286,7 @@ sub last_scan {
                 . "</span>"
             : "<span background = '#FFFFC8'>"
                 . (
-                sprintf gettext('Date of last known infection: %s'),
+                sprintf gettext('Date of last known threat: %s'),
                 $last_infection
                 )
                 . "</span>"
@@ -2406,7 +2330,7 @@ sub get_md5 {
 
 sub about {
     my $about = Gtk2::AboutDialog->new;
-	$about->set_program_name( 'ClamTk' );
+    $about->set_program_name('ClamTk');
     $about->set_authors("Dave M, 2004-2012\ndave.nerd <at> gmail.com");
     $about->set_version( ClamTk::App->get_TK_version() );
     my $contributors = 'Please see http://clamtk.sf.net/credits.html';
