@@ -154,6 +154,10 @@ sub custom_prefs {
         $pkg{'LastInfection'} = gettext('Never');
     }
 
+    if ( !exists $pkg{'PulseMode'} ) {
+        $pkg{'PulseMode'} = 'activity';
+    }
+
     for my $o (
         qw{SaveToLog ScanHidden SizeLimit
         Thorough Recursive Mounted}
@@ -164,7 +168,7 @@ sub custom_prefs {
         }
     }
 
-    for my $p (qw{OpenDNS AVCheck GUICheck TruncateLog}) {
+    for my $p (qw{AVCheck GUICheck TruncateLog DupeDB }) {
         # on by default
         if ( !exists $pkg{$p} ) {
             $pkg{$p} = 1;
@@ -197,11 +201,11 @@ sub get_all_prefs {
 sub legit_key {
     # Sanity check the prefs file's keys.
     my @keys = qw(
-        SizeLimit AVCheck HTTPProxy
-        LastInfection OpenDNS GUICheck
+        SizeLimit HTTPProxy
+        LastInfection GUICheck DupeDB
         TruncateLog SaveToLog
         Whitelist Update ScanHidden
-        Thorough Recursive Mounted
+        Thorough Recursive Mounted PulseMode
         );
     return 1 if ( grep { $_[0] eq $_ } @keys );
 }
@@ -317,7 +321,7 @@ sub restore {
         warn "Can't open restore file for reading: $!\n";
         return -1;
     };
-    binmode( $F, ':utf8' );
+    binmode( $F, ':encoding(UTF-8)' );
 
     while (<$F>) {
         chomp;
@@ -348,7 +352,7 @@ sub restore {
             warn "Can't open restore file for writing: $!\n";
             return -1;
         };
-        binmode( $F, ':utf8' );
+        binmode( $F, ':encoding(UTF-8)' );
 
         if ( scalar( keys %p ) ) {
             for my $e ( keys %p ) {
@@ -364,7 +368,7 @@ sub restore {
             warn "Can't open restore file for writing: $!\n";
             return -1;
         };
-        binmode( $F, ':utf8' );
+        binmode( $F, ':encoding(UTF-8)' );
 
         for my $e ( keys %p ) {
             next if ( $e eq $wanted );
