@@ -23,6 +23,8 @@ use Locale::gettext;
 
 use Glib 'TRUE', 'FALSE';
 
+use open ':encoding(utf8)';
+
 # This should be under /usr/bin, but we'll check anyway.
 my $cmd =
       ( -e '/usr/bin/crontab' )       ? '/usr/bin/crontab'
@@ -327,7 +329,7 @@ sub apply_scan {
     remove('# clamtk-scan');
 
     my $tmp_file = "$paths->{clamtk}" . "/" . "cron";
-    open( my $T, '>:encoding(UTF-8)', $tmp_file ) or do {
+    open( my $T, '>', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -397,7 +399,7 @@ sub apply_scan {
     $full_cmd .= ' --log="$HOME/.clamtk/history/$(date +\%b-\%d-\%Y).log"'
         . ' 2>/dev/null';
 
-    open( $T, '>>:encoding(UTF-8)', $tmp_file ) or do {
+    open( $T, '>>', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -425,7 +427,7 @@ sub apply_defs {
     remove('# clamtk-defs');
 
     my $tmp_file = $paths->{clamtk} . "/" . "cron";
-    open( my $T, '>:encoding(UTF-8)', $tmp_file ) or do {
+    open( my $T, '>', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -465,7 +467,7 @@ sub apply_defs {
         }
     }
 
-    open( $T, '>>:encoding(UTF-8)', $tmp_file ) or do {
+    open( $T, '>>', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -490,7 +492,7 @@ sub remove {
     my $paths = ClamTk::App->get_path('clamtk');
 
     my $tmp_file = "$paths/cron";
-    open( my $T, '>:encoding(UTF-8)', $tmp_file ) or do {
+    open( my $T, '>', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
