@@ -13,7 +13,7 @@ package ClamTk::Submit;
 # This file probably needs some work.  Or I need to understand encoding
 # better.  It's like one big hackjob.
 
-use strict;
+# use strict;
 # use warnings FATAL => 'all';
 $| = 1;
 
@@ -89,7 +89,6 @@ sub analysis {
     $name_entry->signal_connect(
         'insert-text' => sub {
             my ( $widget, $string, $position ) = @_;
-
             if ( $string !~ m#[\w0-9\.\-\_\s]# ) {
                 $name_entry->signal_stop_emission_by_name('insert-text');
             }
