@@ -99,14 +99,6 @@ sub update_dialog {
         $gui_box, gettext('N/A'),
         ];
 
-    # Add option to check for ClamAV engine updates
-    push @{ $update_list->{data} },
-        [
-        gettext('Antivirus engine'),
-        gettext('Check for antivirus engine updates'),
-        $gui_box, gettext('N/A'),
-        ];
-
     # This box will hold buttons to check for updates, cancel,
     # close the window, and if the user's root, warn them.
     my $choicebar = Gtk2::Toolbar->new();
@@ -250,10 +242,6 @@ sub decision {
                     : $value == 5 ? gettext('Check failed')
                     :               '';
                 ClamTk::GUI->set_tk_status($value);
-            } elsif ( $rowref->[0] eq gettext('Antivirus engine') ) {
-                $update_list->{data}[$row][3] = gettext('Checking...');
-                $value = ClamTk::GUI->set_AV_engine_status();
-                $update_list->{data}[$row][3] = $value;
             } else {
                 #warn 'ref = ', $rowref->[0], "\n";
             }
@@ -364,6 +352,11 @@ sub update_signatures {
             Gtk2->main_iteration while ( Gtk2->events_pending );
             $update_list->{data}[$print_row][3] = $final;
             Gtk2->main_iteration while ( Gtk2->events_pending );
+        } elsif ( $line =~ /Update failed/ ) {
+            $final = gettext('Update failed');
+            $update_list->{data}[$print_row][3] = $final;
+            Gtk2->main_iteration while ( Gtk2->events_pending );
+            return -1;
         } else {
             next;
         }
